import { Directive, Host, Inject, Input, } from '@angular/core';
import { ControlContainer, FormGroup } from '@angular/forms';
import { NG_INJECTABLE_CONTROL } from './injectable-control.token';
import * as i0 from "@angular/core";
import * as i1 from "@angular/forms";
/**
 * Work around for ExpressionChangeAfterCheckedError from
 * https://stackoverflow.com/questions/45661010/dynamic-nested-reactive-form-expressionchangedafterithasbeencheckederror
 */
const resolvedPromise = (() => Promise.resolve(null))();
export class InjectControlDirective {
    constructor(controlContainer, host) {
        this.controlContainer = controlContainer;
        this.host = host;
    }
    set disabled(isDisabled) {
        const control = this.host.control;
        if (isDisabled && control.enabled) {
            control.disable();
        }
        else if (!isDisabled && control.disabled) {
            control.enable();
        }
    }
    /**
     * TODO: Use on changes instead and support a case where the input can be
     * changed, similar to how FormControlName and FormControlDirective work.
     */
    ngOnInit() {
        const parent = this.controlContainer.control;
        if (parent instanceof FormGroup) {
            const control = parent.get(this.injectControl);
            if (!control) {
                throw new Error(`No control ${this.injectControl} in control container`);
            }
            this.originalControl = control;
            resolvedPromise.then(() => {
                if (control.value) {
                    this.host.control.patchValue(control.value);
                }
                if (control.validator) {
                    this.host.control.setValidators(control.validator);
                    this.host.control.updateValueAndValidity({
                        onlySelf: true,
                        emitEvent: false,
                    });
                }
                if (control.asyncValidator) {
                    this.host.control.setAsyncValidators(control.asyncValidator);
                    this.host.control.updateValueAndValidity({
                        onlySelf: true,
                        emitEvent: false,
                    });
                }
                parent.setControl(this.injectControl, this.host.control);
                parent.updateValueAndValidity({ emitEvent: false });
            });
        }
        else {
            throw new Error(`Unsupported control container type ${parent === null || parent === void 0 ? void 0 : parent.constructor.name}`);
        }
    }
    ngOnDestroy() {
        const parent = this.controlContainer.control;
        if (parent instanceof FormGroup) {
            resolvedPromise.then(() => {
                if (this.originalControl) {
                    parent.setControl(this.injectControl, this.originalControl);
                    parent.updateValueAndValidity({ emitEvent: false });
                }
            });
        }
    }
}
InjectControlDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.1.0", ngImport: i0, type: InjectControlDirective, deps: [{ token: i1.ControlContainer }, { token: NG_INJECTABLE_CONTROL, host: true }], target: i0.ɵɵFactoryTarget.Directive });
InjectControlDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.1.0", type: InjectControlDirective, selector: "[injectControl]", inputs: { injectControl: "injectControl", disabled: "disabled" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.1.0", ngImport: i0, type: InjectControlDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[injectControl]',
                }]
        }], ctorParameters: function () { return [{ type: i1.ControlContainer }, { type: undefined, decorators: [{
                    type: Host
                }, {
                    type: Inject,
                    args: [NG_INJECTABLE_CONTROL]
                }] }]; }, propDecorators: { injectControl: [{
                type: Input
            }], disabled: [{
                type: Input
            }] } });
//# sourceMappingURL=inject-control.directive.js.map